{
  Solutions to the Advent Of Code.
  Copyright (C) 2023  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UTrebuchetTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, USolver, UBaseTestCases, UTrebuchet;

type

  { TTrebuchetFullDataTestCase }

  TTrebuchetFullDataTestCase = class(TEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
    procedure TestPart2;
  end;

  { TTrebuchetExampleTestCase }

  TTrebuchetExampleTestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
  end;

  { TPart2ExampleTrebuchet }

  TPart2ExampleTrebuchet = class(TTrebuchet)
    function GetDataFileName: string; override;
  end;

  { TTrebuchetPart2ExampleTestCase }

  TTrebuchetPart2ExampleTestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart2;
  end;

implementation

{ TTrebuchetFullDataTestCase }

function TTrebuchetFullDataTestCase.CreateSolver: ISolver;
begin
  Result := TTrebuchet.Create;
end;

procedure TTrebuchetFullDataTestCase.TestPart1;
begin
  AssertEquals(56506, FSolver.GetResultPart1);
end;

procedure TTrebuchetFullDataTestCase.TestPart2;
begin
  AssertEquals(56017, FSolver.GetResultPart2);
end;

{ TTrebuchetExampleTestCase }

function TTrebuchetExampleTestCase.CreateSolver: ISolver;
begin
  Result := TTrebuchet.Create;
end;

procedure TTrebuchetExampleTestCase.TestPart1;
begin
  AssertEquals(142, FSolver.GetResultPart1);
end;

{ TPart2ExampleTrebuchet }

function TPart2ExampleTrebuchet.GetDataFileName: string;
begin
  Result := 'trebuchet_calibration_document2.txt';
end;

{ TTrebuchetPart2ExampleTestCase }

function TTrebuchetPart2ExampleTestCase.CreateSolver: ISolver;
begin
  Result := TPart2ExampleTrebuchet.Create;
end;

procedure TTrebuchetPart2ExampleTestCase.TestPart2;
begin
  AssertEquals(281, FSolver.GetResultPart2);
end;

initialization

  RegisterTest(TTrebuchetFullDataTestCase);
  RegisterTest(TTrebuchetExampleTestCase);
  RegisterTest(TTrebuchetPart2ExampleTestCase);
end.
